<?php


use Typecho\Config;
use Typecho\Cookie;
use Typecho\Db\Exception;
use Typecho\Db\Query;
use Typecho\Router;
use Typecho\Widget\Helper\PageNavigator\Box;
use Widget\Base\Comments;
use Widget\Base\Contents;
use Widget\Contents\From;
use Widget\Options;


/**
 * 评论归档组件
 *
 * @category typecho
 * @package Widget
 * @copyright Copyright (c) 2008 Typecho team (http://www.typecho.org)
 * @license GNU General Public License 2.0
 */

class Handsome_Widget_Comments_Archive extends Widget\Comments\Archive
{
    /**
     * 当前页
     * NOTE: 这个成员和父类中不通用，所以涉及到这个变量函数都需要重写
     * @access private
     * @var integer
     */
    private int $currentPage;

    /**
     * 所有文章个数
     * NOTE: 这个成员和父类中不通用，所以涉及到这个变量函数都需要重写*
     * @access private
     * @var integer
     */
    private int $total = 0;

    /**
     * 子父级评论关系
     * NOTE: 这个成员和父类中不通用，所以涉及到这个变量函数都需要重写
     * @access private
     * @var array
     */
    private array $threadedComments = [];

    public $origin_stack = [];


    public function data(){
        $this->type="comment";
        // 时光机页面的cid
        $this->cid = $this->parameter->parentId;
        $isLogin = Typecho_Widget::widget('Widget_User')->hasLogin();
        $ret = [];
        $this->options->commentsPageDisplay = "first";//主题侧我们固定把较新的显示在前面了！！

        if ('last' == $this->options->commentsPageDisplay) {
            $currentPage = ceil($this->total / $this->options->commentsPageSize)+1;
        } else {
            $currentPage = 0;
        }
        if (version_compare(Utils::getTypeVer(),'1.3.0','>=')) {
            $widget = Widget\Contents\From::allocWithAlias($this->cid, ['cid' => $this->cid]);
            $parent_content = array("permalink"=> $widget->permalink, "pathinfo"=> $widget->path);
        }else {
            $parent_content =  $this->db->fetchRow(Contents::alloc()->select()
                ->where('table.contents.cid = ?', $this->cid)
                ->limit(1), [Contents::alloc(), 'filter']);
        }
        if ($parent_content==null){
            return "[]";
        }
        $path_info = $parent_content['pathinfo'];
        foreach ($this->origin_stack as $key=>$item){

            //计算分页
            if ($key % $this->options->commentsPageSize == 0){
                if ('last' == $this->options->commentsPageDisplay) {
                    $currentPage --;
                } else {
                    $currentPage ++;
                }
            }
            if (strpos($item["text"], '[secret]') !== false) {//提高效率，避免每篇文章都要解析
                if (!$isLogin){
                    continue;
                }
//                $pattern = CommonContent::get_shortcode_regex(array('secret'));
//                $item["text"] = preg_replace_callback("/$pattern/", function ($matches) {
//                    return Content::secretContentParseCallback($matches, true);
//                }, $item["text"]);

            }

            $theId= $this->type . '-' . $item["coid"];;

            if ($this->options->commentsPageBreak) {
                //计算当前属于那一页
                $pageRow = ['permalink' => $path_info, 'commentPage' => $currentPage];
                $permalink = Router::url('comment_page', $pageRow, $this->options->index) . '#' . $theId;
            }else{

                $permalink =  $parent_content['permalink'] . '#' . $theId;

            }

            $ret[] = array(
                "content"=>strip_tags($item["text"]),
                "date"=> $item["date"]->format(DATE_ATOM),
                "path"=> $permalink,
                "coid"=> $item["coid"]
                );
        }




        return json_encode($ret);
    }
    /**
     * 获取查询对象
     *
     * @return Query
     * @throws Exception
     */
    public function select(...$args): Query
    {
        $sql = $this->db->select(
            'table.comments.coid',
            'table.comments.cid',
            'table.comments.author',
            'table.comments.mail',
            'table.comments.url',
            'table.comments.ip',
            'table.comments.authorId',
            'table.comments.ownerId',
            'table.comments.agent',
            'table.comments.text',
            'table.comments.type',
            'table.comments.status',
            'table.comments.parent',
            'table.comments.created',
            'table.comments.stars'
        )
            ->from('table.comments');
        if ($this->parameter->tid!=null) {
            if ($this->parameter->tid==-1){
                $sql = $sql->join(
                    "table.handsomerelationships",
                    "table.handsomerelationships.coid = table.comments.coid",
                    'left')
                    // 注意这里包含了所有parent!=0的评论
                    ->where('table.handsomerelationships.tid is null or table.comments.parent!=0');
            } else if ($this->parameter->tid==-2){
                // 包含[secret 标签的说说
                $sql = $sql->where("table.comments.text like '%[secret]%'");
            } else{
                $sql = $sql->join(
                    "table.handsomerelationships",
                    "table.handsomerelationships.coid = table.comments.coid",
                    'left')
                    // 注意这里包含了所有parent!=0的评论
                    ->where('table.handsomerelationships.tid=? or table.comments.parent!=0', $this->parameter->tid);
            }
        }
        return $sql;
    }
    /**
     * 执行函数
     *
     * @access public
     * @return void
     */
    public function execute()
    {

        if (!$this->parameter->parentId) {
            return;
        }

        $commentsAuthor = Cookie::get('__typecho_remember_author');
        $commentsMail = Cookie::get('__typecho_remember_mail');
        $select = $this->select()->where('table.comments.cid = ?', $this->parameter->parentId)
            ->where(
                'table.comments.status = ? OR (table.comments.author = ?'
                . ' AND table.comments.mail = ? AND table.comments.status = ?)',
                'approved',
                $commentsAuthor,
                $commentsMail,
                'waiting'
            );
        $threadedSelect = null;

        if ($this->options->commentsShowCommentOnly) {
            $select->where('table.comments.type = ?', 'comment');
        }

        $select->order('table.comments.coid', 'ASC');
        $this->db->fetchAll($select, [$this, 'push']);

        /** 需要输出的评论列表 */
        $outputComments = [];


        /** 如果开启评论回复 */
        if ($this->options->commentsThreaded) {
            foreach ($this->stack as $coid => &$comment) {

                /** 取出父节点 */
                $parent = $comment['parent'];

                /** 如果存在父节点 */
                if (0 != $parent && isset($this->stack[$parent])) {

                    /** 如果当前节点深度大于最大深度, 则将其挂接在父节点上 */
                    if ($comment['levels'] >= $this->options->commentsMaxNestingLevels) {
                        $comment['levels'] = $this->stack[$parent]['levels'];
                        $parent = $this->stack[$parent]['parent'];     // 上上层节点
                        $comment['parent'] = $parent;
                    }

                    /** 计算子节点顺序 */
                    $comment['order'] = isset($this->threadedComments[$parent])
                        ? count($this->threadedComments[$parent]) + 1 : 1;

                    /** 如果是子节点 */
                    $this->threadedComments[$parent][$coid] = $comment;
                } else {
                    // $parent ==0 ||  不存在 $this->stack[$parent]
                    // select 查询中包含了所有子评论（即使不匹配tid）
                    // 因此存在parent不为空，但是对应的parent并没有查询到
                    // 因此将这种评论过滤掉
                    if ($parent!=0 && !isset($this->stack[$parent])){
                    }else {
                        $outputComments[$coid] = $comment;
                    }
                }

            }

            $this->stack = $outputComments;
        }

        /** 评论排序 */
        if ('DESC' == $this->options->commentsOrder) {
            $this->stack = array_reverse($this->stack, true);
            $this->threadedComments = array_map('array_reverse', $this->threadedComments);
        }

        /** 评论总数 */
        $this->total = count($this->stack);

        $this->origin_stack = array_values($this->stack);


        /** 对评论进行分页 */
        if ($this->options->commentsPageBreak) {

            if ('last' == $this->options->commentsPageDisplay && !$this->parameter->commentPage) {
                $this->currentPage = ceil($this->total / $this->options->commentsPageSize);
            } else {
                $this->currentPage = $this->parameter->commentPage ? $this->parameter->commentPage : 1;
            }

            /** 截取评论 */
            $this->stack = array_slice(
                $this->stack,
                ($this->currentPage - 1) * $this->options->commentsPageSize,
                $this->options->commentsPageSize
            );

            /** 评论置位 */
            $this->length = count($this->stack);
            $this->row = $this->length > 0 ? current($this->stack) : [];
        }

        reset($this->stack);
    }


    /**
     * 输出分页
     *
     * @access public
     * @param string $prev 上一页文字
     * @param string $next 下一页文字
     * @param int $splitPage 分割范围
     * @param string $splitWord 分割字符
     * @param string|array $template 展现配置信息
     * @return void
     * @throws \Typecho\Widget\Exception
     */
    public function pageNav(
        string $prev = '&laquo;',
        string $next = '&raquo;',
        int $splitPage = 3,
        string $splitWord = '...',
               $template = ''
    ) {
        if ($this->options->commentsPageBreak) {
            $default = [
                'wrapTag'   => 'ol',
                'wrapClass' => 'page-navigator'
            ];

            if (is_string($template)) {
                parse_str($template, $config);
            } else {
                $config = $template ?: [];
            }

            $template = array_merge($default, $config);

            if (version_compare(Utils::getTypeVer(),'1.3.0','>=')) {
                $query = Router::url('comment_page', [
                    'permalink' => $this->parameter->parentContent->path,
                    'commentPage' => '{commentPage}'
                ], $this->options->index);
            }else {
                $pageRow = $this->parameter->parentContent;
                $pageRow['permalink'] = $pageRow['pathinfo'];
                $query = Router::url('comment_page', $pageRow, $this->options->index);
            }
            if ($this->parameter->tid!=null){
                $query.="?tag=".$this->parameter->tid;
            }
            self::pluginHandle()->trigger($hasNav)->pageNav(
                $this->currentPage,
                $this->total,
                $this->options->commentsPageSize,
                $prev,
                $next,
                $splitPage,
                $splitWord,
                $template,
                $query
            );

            if (!$hasNav && $this->total > $this->options->commentsPageSize) {
                /** 使用盒状分页 */
                $nav = new Box($this->total, $this->currentPage, $this->options->commentsPageSize, $query);
                $nav->setPageHolder('commentPage');
                $nav->setAnchor('comments');

                echo '<' . $template['wrapTag'] . (empty($template['wrapClass'])
                        ? '' : ' class="' . $template['wrapClass'] . '"') . '>';
                $nav->render($prev, $next, $splitPage, $splitWord, $template);
                echo '</' . $template['wrapTag'] . '>';
            }
        }
    }

    /**
     * 子评论
     *
     * @return array
     */
    protected function ___children(): array
    {
        return $this->options->commentsThreaded && !$this->isTopLevel && isset($this->threadedComments[$this->coid])
            ? $this->threadedComments[$this->coid] : [];
    }

    protected function ___commentPage(): int
    {
        return $this->currentPage;
    }

}
